@@ -22,129 +22,27 @@
 
 #include "version.h"
 
+#include "base/error_state.h"
+#include "base/nested_output.h"
 #include "persistent-data/space-maps/core.h"
+#include "persistent-data/file_utils.h"
 #include "thin-provisioning/device_tree.h"
-#include "thin-provisioning/file_utils.h"
 #include "thin-provisioning/mapping_tree.h"
 #include "thin-provisioning/superblock.h"
 
+using namespace base;
 using namespace std;
 using namespace thin_provisioning;
 
 //----------------------------------------------------------------
 
 namespace {
-
-	class end_message {};
-
-	class nested_output {
-	public:
-		nested_output(ostream &out, unsigned step)
-			: out_(out),
-			  step_(step),
-			  beginning_of_line_(true),
-			  enabled(true),
-			  indent_(0) {
-		}
-
-		template <typename T>
-		nested_output &operator <<(T const &t) {
-			if (beginning_of_line_) {
-				beginning_of_line_ = false;
-				indent();
-			}
-
-			if (enabled)
-				out_ << t;
-
-			return *this;
-		}
-
-		nested_output &operator <<(end_message const &m) {
-			beginning_of_line_ = true;
-
-			if (enabled)
-				out_ << endl;
-
-			return *this;
-		}
-
-		void inc_indent() {
-			indent_ += step_;
-		}
-
-		void dec_indent() {
-			indent_ -= step_;
-		}
-
-		struct nest {
-			nest(nested_output &out)
-			: out_(out) {
-				out_.inc_indent();
-			}
-
-			~nest() {
-				out_.dec_indent();
-			}
-
-			nested_output &out_;
-		};
-
-		nest push() {
-			return nest(*this);
-		}
-
-		void enable() {
-			enabled = true;
-		}
-
-		void disable() {
-			enabled = false;
-		}
-
-	private:
-		void indent() {
-			if (enabled)
-				for (unsigned i = 0; i < indent_; i++)
-					out_ << ' ';
-		}
-
-		ostream &out_;
-		unsigned step_;
-
-		bool beginning_of_line_;
-		bool enabled;
-		unsigned indent_;
-	};
-
-	//--------------------------------
-
-	enum error_state {
-		NO_ERROR,
-		NON_FATAL,	// eg, lost blocks
-		FATAL		// needs fixing before pool can be activated
-	};
-
-	error_state
-	combine_errors(error_state lhs, error_state rhs) {
-		switch (lhs) {
-		case NO_ERROR:
-			return rhs;
-
-		case NON_FATAL:
-			return (rhs == FATAL) ? FATAL : lhs;
-
-		default:
-			return lhs;
-		}
-	}
-
 	//--------------------------------
 
 	block_manager<>::ptr
 	open_bm(string const &path) {
 		block_address nr_blocks = get_nr_blocks(path);
-		typename block_io<>::mode m = block_io<>::READ_ONLY;
+		block_io<>::mode m = block_io<>::READ_ONLY;
 		return block_manager<>::ptr(new block_manager<>(path, nr_blocks, 1, m));
 	}
 
@@ -168,13 +66,13 @@ namespace {
 		virtual void visit(superblock_detail::superblock_corruption const &d) {
 			out_ << "superblock is corrupt" << end_message();
 			{
-				auto _ = out_.push();
+				nested_output::nest _ = out_.push();
 				out_ << d.desc_ << end_message();
 			}
 			err_ = combine_errors(err_, FATAL);
 		}
 
-		error_state get_error() const {
+		base::error_state get_error() const {
 			return err_;
 		}
 
@@ -195,7 +93,7 @@ namespace {
 		virtual void visit(device_tree_detail::missing_devices const &d) {
 			out_ << "missing devices: " << d.keys_ << end_message();
 			{
-				auto _ = out_.push();
+				nested_output::nest _ = out_.push();
 				out_ << d.desc_ << end_message();
 			}
 
@@ -216,13 +114,14 @@ namespace {
 	class mapping_reporter : public mapping_tree_detail::damage_visitor {
 	public:
 		mapping_reporter(nested_output &out)
-		: out_(out) {
+		: out_(out),
+		  err_(NO_ERROR) {
 		}
 
 		virtual void visit(mapping_tree_detail::missing_devices const &d) {
 			out_ << "missing all mappings for devices: " << d.keys_ << end_message();
 			{
-				auto _ = out_.push();
+				nested_output::nest _ = out_.push();
 				out_ << d.desc_ << end_message();
 			}
 			err_ = combine_errors(err_, FATAL);
@@ -231,7 +130,7 @@ namespace {
 		virtual void visit(mapping_tree_detail::missing_mappings const &d) {
 			out_ << "thin device " << d.thin_dev_ << " is missing mappings " << d.keys_ << end_message();
 			{
-				auto _ = out_.push();
+				nested_output::nest _ = out_.push();
 				out_ << d.desc_ << end_message();
 			}
 			err_ = combine_errors(err_, FATAL);
@@ -267,7 +166,7 @@ namespace {
 
 		out << "examining superblock" << end_message();
 		{
-			auto _ = out.push();
+			nested_output::nest _ = out.push();
 			check_superblock(bm, sb_rep);
 		}
 
@@ -280,7 +179,7 @@ namespace {
 		if (fs.check_device_tree) {
 			out << "examining devices tree" << end_message();
 			{
-				auto _ = out.push();
+				nested_output::nest _ = out.push();
 				device_tree dtree(tm, sb.device_details_root_,
 						  device_tree_detail::device_details_traits::ref_counter());
 				check_device_tree(dtree, dev_rep);
@@ -290,7 +189,7 @@ namespace {
 		if (fs.check_mapping_tree_level1 && !fs.check_mapping_tree_level2) {
 			out << "examining top level of mapping tree" << end_message();
 			{
-				auto _ = out.push();
+				nested_output::nest _ = out.push();
 				dev_tree dtree(tm, sb.data_mapping_root_,
 					       mapping_tree_detail::mtree_traits::ref_counter(tm));
 				check_mapping_tree(dtree, mapping_rep);
@@ -299,7 +198,7 @@ namespace {
 		} else if (fs.check_mapping_tree_level2) {
 			out << "examining mapping tree" << end_message();
 			{
-				auto _ = out.push();
+				nested_output::nest _ = out.push();
 				mapping_tree mtree(tm, sb.data_mapping_root_,
 						   mapping_tree_detail::block_traits::ref_counter(tm->get_sm()));
 				check_mapping_tree(mtree, mapping_rep);
@@ -345,6 +244,12 @@ int main(int argc, char **argv)
 {
 	int c;
 	flags fs;
+	fs.check_device_tree = true;
+	fs.check_mapping_tree_level1 = true,
+	fs.check_mapping_tree_level2 = true,
+	fs.ignore_non_fatal_errors = false,
+	fs.quiet = false;
+
 	char const shortopts[] = "qhV";
 	option const longopts[] = {
 		{ "quiet", no_argument, NULL, 'q'},
@@ -356,12 +261,6 @@ int main(int argc, char **argv)
 		{ NULL, no_argument, NULL, 0 }
 	};
 
-	fs.check_device_tree = true;
-	fs.check_mapping_tree_level1 = true;
-	fs.check_mapping_tree_level2 = true;
-	fs.ignore_non_fatal_errors = false;
-	fs.quiet = false;
-
 	while ((c = getopt_long(argc, argv, shortopts, longopts, NULL)) != -1) {
 		switch(c) {
 		case 'h':
